import { PromiseDelegate } from '@lumino/coreutils';
import { MainAreaWidget } from '@jupyterlab/apputils';
import { requestAPI } from './handler';
import { CodeViewerWidget } from './widgets/CodeViewer';
import { notebookIcon } from '@jupyterlab/ui-components';
export class TransformPollingManager {
    constructor(props) {
        this._ready = new PromiseDelegate();
        this._mutableAI = null;
        this._count = 0;
        this._file_path = '';
        this._current_file_path = '';
        this._docManager = props.docManager;
        this._app = props.app;
        props
            .getSettings()
            .then(mutableAI => {
            this._mutableAI = mutableAI;
            this._ready.resolve();
        })
            .catch(reason => {
            console.warn(reason);
            this._ready.reject(reason);
        });
        this._count = 0;
        this._file_path = '';
        this.startPolling = this.startPolling.bind(this);
        this.acceptFile = this.acceptFile.bind(this);
        this.declineFile = this.declineFile.bind(this);
    }
    /**
     * A promise that resolves when the settings have been loaded.
     */
    get ready() {
        return this._ready.promise;
    }
    async startPolling(file_path, current_file_path) {
        this._file_path = file_path;
        this._current_file_path = current_file_path;
        const dataToSend = { file_path };
        const pollingRef = setInterval(async () => {
            this._count += 1;
            try {
                const response = await requestAPI('CHECK_STATUS', {
                    body: JSON.stringify(dataToSend),
                    method: 'POST'
                });
                if ((response === null || response === void 0 ? void 0 : response.status) === 'finished') {
                    // Open transformed file here.
                    const close = () => { var _a; return (_a = this._app.shell.currentWidget) === null || _a === void 0 ? void 0 : _a.close(); };
                    const content = new CodeViewerWidget(close, response.file, this._app, {
                        onAcceptChanges: this.acceptFile,
                        onDeclineChanges: this.declineFile
                    });
                    const widget = new MainAreaWidget({ content });
                    widget.title.label = 'MutableAI Code Viewer';
                    widget.title.icon = notebookIcon;
                    this._app.shell.add(widget, 'main', { mode: 'split-right' });
                    clearInterval(pollingRef);
                }
                else {
                    if (this._count >= 60) {
                        this._count = 0;
                        this._file_path = '';
                        clearInterval(pollingRef);
                    }
                }
            }
            catch (error) {
                console.log('Failed to make request. Reason: ' + error.toString());
            }
        }, 1000);
    }
    acceptFile() {
        var _a, _b;
        const apiKey = (_a = this._mutableAI) === null || _a === void 0 ? void 0 : _a.get('apiKey').composite;
        const transformDomain = (_b = this._mutableAI) === null || _b === void 0 ? void 0 : _b.get('transformDomain').composite;
        const dataToSend = {
            file_path: this._file_path,
            current_file_path: this._current_file_path,
            url: transformDomain,
            action: 'accept',
            apiKey
        };
        const reply = requestAPI('FILE_ACTION', {
            body: JSON.stringify(dataToSend),
            method: 'POST'
        });
        reply
            .then((response) => {
            if (response.status === 'completed') {
                console.log('File accepted.');
                const widgets = this._app.shell.widgets('main');
                while (true) {
                    const widget = widgets.next();
                    if (widget) {
                        widget.close();
                    }
                    else {
                        break;
                    }
                }
                setTimeout(() => {
                    this._docManager.open(this._current_file_path);
                    const launcher = this._app.shell.widgets('main').next();
                    if ((launcher === null || launcher === void 0 ? void 0 : launcher.title.label) === 'Launcher') {
                        setTimeout(() => {
                            launcher.close();
                        }, 300);
                    }
                }, 300);
            }
            else {
                console.log('File accepting failed.');
            }
        })
            .catch(e => console.log('File accepting failed.', e));
    }
    declineFile() {
        var _a, _b;
        const apiKey = (_a = this._mutableAI) === null || _a === void 0 ? void 0 : _a.get('apiKey').composite;
        const transformDomain = (_b = this._mutableAI) === null || _b === void 0 ? void 0 : _b.get('transformDomain').composite;
        const dataToSend = {
            file_path: this._file_path,
            current_file_path: this._current_file_path,
            action: 'decline',
            url: transformDomain,
            apiKey
        };
        const reply = requestAPI('FILE_ACTION', {
            body: JSON.stringify(dataToSend),
            method: 'POST'
        });
        reply
            .then((response) => {
            if (response.status === 'completed') {
                console.log('File decline.');
                const widgets = this._app.shell.widgets('main');
                while (true) {
                    const widget = widgets.next();
                    if (widget) {
                        if (widget.title.label === 'MutableAI Code Viewer') {
                            widget.close();
                        }
                    }
                    else {
                        break;
                    }
                }
            }
            else {
                console.log('File declining failed.');
            }
        })
            .catch(e => console.log('File declining failed.', e));
    }
}
